// Contains class definitions
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Artistic License, included as the file
// "LICENSE" in the source code archive.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// You should have received a copy of the file "LICENSE", containing
// the License John Weiss originally placed this program under.
#include "QMeter.h"
#include <iostream.h>
#include <qpainter.h>



// ***************
//  Here, we define the actual functions



//||||||||||||||||||||||||||||||||||||||||||||||||
//
//  Class QMeter
//
//||||||||||||||||||||||||||||||||||||||||||||||||



// Constructor for the class
//
QMeter::QMeter( QWidget* parent, const char* name, WFlags f ) 
  : QLabel(parent,name,f) 
{
  setAlignment(AlignCenter);
  resize(5,20);
  oldSize = size();
  lightsize = 0.1;
  initMeter();
  setLevel(0.5);
}


// Redraws the meter
//
void QMeter::Redraw(void)
{
   int sepsize, seppos;
   int nlines, endheight, sepheight;
   int widthmark,heightmark;
   int i;
   bool paint_ready;
   QPainter p;
   
   
   // Some initial settings
   widthmark = meter_map.width();
   heightmark = meter_map.height();
   if( (widthmark<1) || (heightmark<1) )
	 {
	 cerr << "Fatal error in QMeter::setLevel()." << endl;
	 cerr << "Improper width or height specification." << endl;
	 cerr << "Aborting program." << endl;
	 exit(-1);
	 }
   
   // Check to see if we separate the lights
   if(continuous)
	 {
	 sepsize = 0;
	 nlines = 0;
	 endheight = (int)(heightmark*current_level);
	 }
   else
	 {
	 sepsize = 1 + heightmark/100;
	 nlines = (int)(heightmark*current_level)/ppl;
	 sepheight = sepsize*nlines;
	 endheight = (int)(heightmark*(1.0-1.0/ppl)*current_level) + 
	    sepheight;
	 if(endheight > heightmark)
	    endheight = heightmark;
	 }
   
   // Draw the meter block
   meter_map.fill( backgroundColor() );
   paint_ready = p.begin( &meter_map );
   if( !paint_ready )
	 {
	 cerr << "Fatal error in QMeter::setLevel()." << endl;
	 cerr << "QPainter::begin() failed." << endl;
	 cerr << "Aborting program." << endl;
	 exit(-2);
	 }
   p.setBackgroundColor( backgroundColor() );
   p.setPen( foregroundColor() );
   p.setBrush( foregroundColor() );
   p.drawRect(flipVert(1,1,widthmark,endheight,meter_map));
   
   // Now draw the separators
   p.setPen( backgroundColor() );
   p.setBrush( backgroundColor() );
   for(i=1; i<=nlines; i++)
	 {
	 seppos = i*ppl;//-sepsize;
	 p.drawRect(flipVert(1,seppos,widthmark,sepsize,meter_map));
	}
   
   // Done
   p.end();
   setPixmap(meter_map);
}


// Sets the meter to "percent", which is the percent of the
// total meter size.
//
void QMeter::setLevel(double percent)
{
  // Make sure the paramenter is in bounds
  if(percent < 0.0)
	{
	percent *= -1;
	}
  
  if(percent > 1.0)
	{
	percent = 1.0;
	}

  current_level = percent;
  Redraw();
}
//
void QMeter::setLevel(int percent)
{
  setLevel( (float)percent/100.0 );
}



//||||||||||||||||||||||||||||||||
//
// End
//
